/*
Example for Barcode Click

    Date          : Apr 2018.
    Author        : Milos Vidojevic

Test configuration STM32 :

    MCU              : STM32F407VG
    Dev. Board       : Clicker 2 STM32
    ARM Compiler ver : v6.0.0.0

---

Description :

  Simple remote barcode scanner demo application with two devices 
  (development systems) :
  
    - Display node which carries :
      - 8x8 click on mikroBUS 1
      - tRF click on mikroBUS 2
      
    - Scanner node which carries : 
      - Barcode click on mikroBUS 1
      - tRF click on mikroBUS 2

  Display node acts as master which will send request for barcode scanning to 
  the scanner node which acts as slave. Request is sent by the master by clickng
  the on board button 1. After the request is received by the slave it will turn 
  on barcode scanner and search for valid barcode. After successful scanning the
  slave send back scanned information back to master which will display received
  information on the 8x8 matrix.
  
  Note :

    Use PLD file before recompilation of project for the particular node.

*/

#ifdef DEMO_SCANNER
#include "Click_Barcode_types.h"
#include "Click_Barcode_config.h"
#endif
#ifdef DEMO_DISPLAY
#include "Click_8x8_R_config.h"
#include "Click_8x8_R_types.h"
#endif

uint8_t counter;
uint8_t barcode_buffer[ 255 ];

#ifdef DEMO_DISPLAY
uint8_t display_buffer[ 255 ];
#endif

void systemInit()
{
    UART3_Init_Advanced(19200, _UART_8_BIT_DATA, _UART_NOPARITY, _UART_ONE_STOPBIT, &_GPIO_MODULE_USART3_PD89);

    RXNEIE_USART3_CR1_bit = 1;
    NVIC_IntEnable( IVT_INT_USART3 );
    
#ifdef DEMO_DISPLAY
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_CS_PIN, _GPIO_OUTPUT );
    mikrobus_spiInit( _MIKROBUS1, &_C8X8R_SPI_CFG[0] );
    GPIO_Digital_Input( &GPIOE_BASE, _GPIO_PINMASK_0 );
#endif
#ifdef DEMO_SCANNER
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_RST_PIN, _GPIO_OUTPUT );
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_PWM_PIN, _GPIO_OUTPUT );
    mikrobus_uartInit( _MIKROBUS1, &_BARCODE_UART_CFG[0] );

    RXNEIE_USART2_CR1_bit = 1;
    NVIC_IntEnable( IVT_INT_USART2 );
#endif

    EnableInterrupts();
    Delay_ms( 100 );
}

void applicationInit()
{
    counter = 0;
    memset( barcode_buffer, 0, 255 );
    memset( display_buffer, 0, 255 );
    
#ifdef DEMO_DISPLAY
    c8x8r_spiDriverInit( (T_C8X8R_P)&_MIKROBUS1_GPIO, (T_C8X8R_P)&_MIKROBUS1_SPI );
    c8x8r_writeCmd(_C8X8R_DECODE_MODE_REG, _C8X8R_NO_DECODE);
    c8x8r_writeCmd(_C8X8R_INTENSITY_REG,   _C8X8R_INTENSITY_15);
    c8x8r_writeCmd(_C8X8R_SCAN_LIMIT_REG,  _C8X8R_DISPLAY_DIGIT_0_7);
    c8x8r_writeCmd(_C8X8R_SHUTDOWN_REG,    _C8X8R_NORMAL_OPERATION);
    c8x8r_setSpeedScroll(_C8X8R_SPEED_MEDIUM);
    c8x8r_displayRefresh();
#endif
#ifdef DEMO_SCANNER
    barcode_uartDriverInit( (T_BARCODE_P)&_MIKROBUS1_GPIO, (T_BARCODE_P)&_MIKROBUS1_UART );
    barcode_enableReset( 1 );
#endif
    
    Delay_ms( 100 );
}

void applicationTask()
{
#ifdef DEMO_DISPLAY
    if (Button(&GPIOE_IDR, 0, 100, 0))
    {
        counter = 0;
        memset( barcode_buffer, 0, 255 );
        
        UART3_Write_Text( "SCAN" );
        Delay_ms( 1500 );
        
        if (counter != 0)
        {
            barcode_buffer[ counter ] = 0;
            strcpy( display_buffer + 2, barcode_buffer );
            display_buffer[ 0 ] = 32;
            display_buffer[ 1 ] = 32;
            
            c8x8r_displayString( display_buffer );
            c8x8r_displayRefresh();
        }
    }
#endif
#ifdef DEMO_SCANNER
    if (counter != 0)
    {
        if (strcmp( barcode_buffer, "SCAN" ))
        {
            barcode_enableScaning(_BARCODE_LOGIC_ON);
            Delay_ms( 3000 );
            barcode_enableScaning(_BARCODE_LOGIC_OFF);

            memset( barcode_buffer, 0, 255 );
            counter = 0;
        }
    }
#endif
}

void main()
{
    systemInit();
    applicationInit();
    
    while (1)
    {
        applicationTask();
    }
}

void TRF_ISR() iv IVT_INT_USART3 ics ICS_AUTO
{
    if( RXNE_USART3_SR_bit )
    {
        barcode_buffer[ counter++ ] = USART3_DR;
    }
}

#ifdef DEMO_SCANNER
void BARCODE_ISR() iv IVT_INT_USART2 ics ICS_AUTO
{
    if( RXNE_USART2_SR_bit )
    {
        UART3_Write( USART2_DR );
    }
}
#endif